<?php

namespace Drupal\Tests\field_context\Functional;

use Drupal\Core\Language\LanguageInterface;
use Drupal\Tests\block\Traits\BlockCreationTrait;
use Drupal\Tests\field\Traits\EntityReferenceFieldCreationTrait;
use Drupal\Tests\taxonomy\Functional\Views\TaxonomyTestBase;
use Drupal\taxonomy\Entity\Term;
use Drupal\views\Tests\ViewTestData;
use Drupal\views\Views;

/**
 * Tests Field Context view plugin.
 *
 * @group field_context
 */
class FieldContextTest extends TaxonomyTestBase {

  use BlockCreationTrait;
  use EntityReferenceFieldCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'field_context',
    'node',
    'views',
    'taxonomy',
    'field_context_test_views',
  ];

  /**
   * Views used by this test.
   *
   * @var array
   */
  public static array $testViews = [
    'test_field_context',
  ];

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function setUp($import_test_views = TRUE, $modules = []): void {
    parent::setUp($import_test_views, $modules);

    $modules[] = 'field_context_test_views';
    if ($import_test_views) {
      ViewTestData::createTestViews(static::class, $modules);
    }

    $settings = [
      'name' => $this->randomMachineName(),
      'description' => $this->randomMachineName(),
      // Use the first available text format.
      'format' => 'plain_text',
      'vid' => $this->vocabulary->id(),
      'langcode' => LanguageInterface::LANGCODE_NOT_SPECIFIED,
    ];
    $term = Term::create($settings);
    $term->save();

    // Clear default nodes list.
    $this->nodes = [];

    $node = [];
    $node['type'] = 'article';
    $node['field_views_testing_tags'][]['target_id'] = $term->id();
    $this->nodes[] = $this->drupalCreateNode($node);
    $this->nodes[] = $this->drupalCreateNode($node);
    $this->nodes[] = $this->drupalCreateNode($node);

    $this->drupalPlaceBlock("views_block:test_field_context-block_1");
  }

  /**
   * Test the overall Field Context module.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   */
  public function testFieldContext(): void {
    $assert = $this->assertSession();

    // Make sure view exists.
    $view = Views::getView('test_field_context');
    $this->assertNotNull($view);

    // Make sure the setup did create nodes.
    $this->assertNotEmpty($this->nodes);
    $first = $this->nodes[0];
    $second = $this->nodes[1];
    $third = $this->nodes[2];

    // Go to the first page.
    $this->drupalGet('node/' . $first->id());
    $assert->statusCodeEquals(200);

    // Assuming the block was placed correctly we should see the title of the
    // other two nodes.
    $assert->pageTextContains($second->label());
    $assert->pageTextContains($third->label());
  }

}
